#!/bin/sh

# TRIPsql Engine installer script for Unix platforms.
#
#  Created : 2004-09-16
#  By      : Tomas Fors, TietoEnator Media Systems AB, Sweden
#
#  Prior to running this script:
#       1 - create a directory where TRIPsql will be installed
#       2 - change directory to the directory just created
#       3 - tar the TRIPsql files off the distribution media
#	4 - change directory to the <version>/bin directory
#	5 - run install: ./install.sh
#

VER_MAJOR=2
VER_MINOR=0
VER_PATCH=1

VERS_DIR=v$VER_MAJOR$VER_MINOR$VER_PATCH
VERSION=v$VER_MAJOR.$VER_MINOR-$VER_PATCH

echo ""
echo " **************  TRIPsql installation script $VERSION  **************"
echo ""
echo ""

WHOM=`(whoami) 2> /dev/null`
if [ $? != "0" ] ; then
  if [ -f /usr/ucb/whoami ] ; then
    WHOM=`/usr/ucb/whoami`
  fi
fi

if [ "$WHOM" != root ]; then
   echo "You must be root to run this script..."
   echo ""
   exit 1
fi

if [ ! -f `pwd`/install.sh ]; then
   echo Error: Current directory must be the one in which this script is located.
   echo ""
   exit 1
fi

CONFIGFILE=/usr/local/trip/sys/conf/tdbs.conf
if [ ! -f $CONFIGFILE ]; then
   CONFIGFILE=/.TRIPrcs
fi

# Make sure we have write access to the config file
if [ ! -w $CONFIGFILE ]; then
   echo Error: $CONFIGFILE must be writable.
   echo ""
   exit 1
fi

TDBS_EXE=`grep TDBS_EXE $CONFIGFILE | ( read r; echo $r | cut -d= -f2 | tr -d '\r')`

# Verify TRIPsystem presence
if [ ! -d "$TDBS_EXE" ]; then
   echo "Error: Cannot locate TRIPsystem installation."
   echo "       TDBS_EXE is not a directory ($TDBS_EXE)."
   echo ""
   exit 1
fi

# Verify minimum TRIPsystem version
TRIPVERSION=`$TDBS_EXE/trip --version 2> /dev/null`
if [ "$TRIPVERSION" = "" ]; then
   echo "Error: TRIPsystem version must be at least 6.2!"
   echo ""
   exit 1
fi
TRIPVER_MAJOR=`echo $TRIPVERSION | cut -d. -f1`
TRIPVER_MINOR=`echo $TRIPVERSION | cut -d. -f2 | cut -d- -f1`
if [ $TRIPVER_MAJOR -gt 6 ]; then
   echo "Your TRIP version is $TRIPVER_MAJOR.$TRIPVER_MINOR. While this should be ok, please check documentation"
   echo "for any compatibility issues."
   echo ""
elif [ $TRIPVER_MAJOR -eq 6 ] && [ $TRIPVER_MINOR -ge 2 ]; then
   echo "Your TRIP version is $TRIPVER_MAJOR.$TRIPVER_MINOR (compatible)."
   echo ""
else
   echo "Error: Your TRIP version is $TRIPVER_MAJOR.$TRIPVER_MINOR (incompatible)."
   echo "       TRIPsystem version must be at least 6.2!"
   echo ""
   exit 1
fi

grep sqlengine $CONFIGFILE > /dev/null
if [ $? -eq 0 ]; then
   IS_UPDATE=1
else
   IS_UPDATE=0
fi

if [ $IS_UPDATE -eq 1 ]; then

   SQL_LIBSYMBOL=`grep sqlengine $CONFIGFILE | grep -v "^#" | cut -d= -f1`
   SQL_LIBPATH=`$TDBS_EXE/queryrcs $SQL_LIBSYMBOL | cut -d\" -f2`
   SQL_BINDIR=`dirname $SQL_LIBPATH`

   # If old installation is broken, there is no point in asking
   # for permission to disable it.
   if [ -f $SQL_LIBPATH ]; then

      if [ "$SQL_BINDIR" = "`cd ../bin; pwd`" ]; then
         echo This module has already been installed.
         echo Do you want to reinstall? [N]
         read YESNO
         if [ "$YESNO" = "" ]; then
            YESNO=N
         fi
         case `echo $YESNO | cut -c1` in
         y|Y) echo "Proceeding..." 
              echo ""
              ;;

         *)   echo ABORT
              exit 1;;
         esac
      fi

      echo Existing TRIPsql installation in `cd $SQL_BINDIR/..; pwd` will be disabled. 
      echo OK? [Y]
      read YESNO
      if [ "$YESNO" = "" ]; then
         YESNO=Y
      fi
      case `echo $YESNO | cut -c1` in
         y|Y) echo "Proceeding..." 
              ;;

         *)   echo ABORT
              exit 1;;
      esac
   else
      echo New installation
   fi
else
   SQL_LIBSYMBOL=TRIPSQL_ENGINE
   echo New installation
fi

SQL_INSTALLDIR=`cd ..; pwd`
SQL_BINDIR=$SQL_INSTALLDIR/bin
  
# Find filters library
if [ ! -d ../bin ]; then
   SQL_INSTALLDIR=`cd ..; pwd`
   echo "Error: bin directory does not exist!"
   exit 3
fi

cd ../bin
SQL_LIB=`find . -type f -name '*sqlengine*' -print|cut -c3-`
if [ "$SQL_LIB" = "" ]; then
   echo "Error: Could not find SQL engine library!"
   echo ""
   exit 2
fi

SQL_LIBPATH=`pwd`/$SQL_LIB

#if [ "`uname`" = "Linux" ]; then
#   FINDDEPTH="-maxdepth 1"
#else
   FINDDEPTH=
#fi

# Set the umask for the duration of the script
OUMASK=`umask`
umask 022

# change user ownership

doloop=0
while [ "$doloop" = "0" ]
do
  echo ""
  echo "Which user should own the TRIPsql files [root]? $C"
  read ans
  case $ans in
  \?)
     echo ""
     echo ""
     echo "   Enter the name of the user who should own all files"
     echo "   and directories in the $VERS_DIR directory structure."
     echo ""
     echo "   The default is root."
     echo ""
     ;;
  *)
     if [ "$ans" = "" ]; then
       owner=root
     else
       owner=$ans
     fi
     chown -R $owner $SQL_INSTALLDIR
     if [ $? -eq 0 ]; then
        doloop=1
     fi
  esac
done
echo "User $owner given ownership of the TRIPsql files" 

# change group ownership

doloop=0
while [ "$doloop" = "0" ]
do
  echo ""
  echo "Which group should own the TRIPsql files? $C"
  read ans
  case $ans in
  \?)
     echo ""
     echo ""
     echo "   Enter the name of the group that should own all files"
     echo "   and directories in the $VERS_DIR directory structure."
     echo ""
     ;;
  "")
     ;;
  *)
     owngroup=$ans
     chgrp -R $owngroup $SQL_INSTALLDIR
     if [ $? -eq 0 ]; then
        doloop=1
     fi
  esac
done
echo "Group $owngroup given group ownership of the TRIPsql files" 

if [ -d $SQL_BINDIR ]; then
   chmod 755 $SQL_BINDIR/*
fi
if [ -d ../bin ]; then
   chmod 755 ../bin/* 
fi


if [ -f /tmp/sql2rcs ]; then
   rm -f /tmp/sql2rcs
   touch /tmp/sql2rcs
fi

if [ -f ../TRIPrcs.mods ]; then
   rm -f ../TRIPrcs.mods
   touch ../TRIPrcs.mods
fi


echo ""
echo Modifying $CONFIGFILE ...

cat $CONFIGFILE | (

   IFS=\\
   while read LINE; do

      # Modify existing SQL Engine library symbol
      RCSSYM=`echo $LINE|cut -d= -f1`
      if [ "$RCSSYM" = "$SQL_LIBSYMBOL" ]; then
         echo "$SQL_LIBSYMBOL=$SQL_LIBPATH" >> /tmp/sql2rcs
         echo "$SQL_LIBSYMBOL=$SQL_LIBPATH" >> ../TRIPrcs.mods
         continue
      fi

      # Check if we should override or keep old TRIPSQL_BASES directory
      if [ "$RCSSYM" = "TRIPSQL_BASES" ]; then
         TRIPSQL_BASES=`echo $LINE|cut -d= -f2`
         echo ""
         if [ ! -d "$TRIPSQL_BASES" ]; then
            # TRIPSQL_BASES setting exist but does not point to an
            # existing directory.
            echo "NOTE: Replacing old, non-existing TRIPSQL_BASES directory '$TRIPSQL_BASES'"
            continue
         fi
         ls $TRIPSQL_BASES/*.BAF > /dev/null 2> /dev/null
         if [ $? -ne 0 ] && [ "$TRIPSQL_BASES" != "$SQL_INSTALLDIR/bases" ] ; then
            # TRIPSQL_BASES directory exist but is empty.
            echo "NOTE: Replacing old TRIPSQL_BASES directory '$TRIPSQL_BASES'"
            continue
         else
            # TRIPSQL_BASES directory exist and contains files
            echo "NOTE: Keeping old TRIPSQL_BASES directory '$TRIPSQL_BASES'"            
         fi
   fi

      # Modify existing TDBS_ASELIBS setting
      if [ "$RCSSYM" = "TDBS_ASELIBS" ]; then
         ASELIBS1=`$TDBS_EXE/queryrcs TDBS_ASELIBS|awk -F\" '{print $2}'`
         ASELIBS2=`$TDBS_EXE/queryrcs TDBS_ASELIBS|awk -F\" '{print $2}'|sed s/$SQL_LIBSYMBOL//`
         if [ "$ASELIBS1" = "$ASELIBS2" ]; then
            ASELIBS1=`echo $ASELIBS1 | cut -d" " -f1`
            if [ "$ASELIBS1" = "" ]; then
               ASELIBS=$SQL_LIBSYMBOL
            else
               ASELIBS=`echo $SQL_LIBSYMBOL,$ASELIBS1`
            fi
         else
            if [ "$ASELIBS2" = "" ]; then
               ASELIBS=$SQL_LIBSYMBOL
            else
               ASELIBS=$ASELIBS1
            fi
         fi

         echo "TDBS_ASELIBS=$ASELIBS" >> /tmp/sql2rcs
         echo "TDBS_ASELIBS=$ASELIBS" >> ../TRIPrcs.mods
         continue
      fi

      # Just perform a copy of all other lines 
      echo $LINE >> /tmp/sql2rcs
   done
)

if [ "`uname`" = "Linux" ]; then
   SILENTFLAG=-silent
else
   SILENTFLAG=-s
fi

# SQL Engine library symbol was not found. Add it.
grep -v "^#" /tmp/sql2rcs | grep "^$SQL_LIBSYMBOL" > /dev/null
if [ $? -ne 0 ]; then
   echo "$SQL_LIBSYMBOL=$SQL_LIBPATH" >> /tmp/sql2rcs
   echo "$SQL_LIBSYMBOL=$SQL_LIBPATH" >> ../TRIPrcs.mods
fi

# ASELIBS setting was not found. Add it.
grep -v "^#" /tmp/sql2rcs | grep TDBS_ASELIBS | grep $SQL_LIBSYMBOL > /dev/null
if [ $? -ne 0 ]; then
   echo "TDBS_ASELIBS=$SQL_LIBSYMBOL" >> /tmp/sql2rcs
   echo "TDBS_ASELIBS=$SQL_LIBSYMBOL" >> ../TRIPrcs.mods
fi

grep TRIPSQL_LOG /tmp/sql2rcs > /dev/null
if [ $? -ne 0 ]; then
   echo "#TRIPSQL_LOG=4" >> /tmp/sql2rcs
   echo "#TRIPSQL_LOG=4" >> ../TRIPrcs.mods
fi

grep -v "^#" /tmp/sql2rcs | grep TRIPSQL_BASES  > /dev/null
if [ $? -ne 0 ]; then
   echo ""
   echo "NOTE: TRIPSQL_BASES location is $SQL_INSTALLDIR/bases"
   echo "TRIPSQL_BASES=$SQL_INSTALLDIR/bases" >> /tmp/sql2rcs
   echo "TRIPSQL_BASES=$SQL_INSTALLDIR/bases" >> ../TRIPrcs.mods
fi

grep SQL_BATCHINDEX /tmp/sql2rcs > /dev/null
if [ $? -ne 0 ]; then
   echo "SQL_BATCHINDEX=0" >> /tmp/sql2rcs
   echo "SQL_BATCHINDEX=0" >> ../TRIPrcs.mods
fi

grep SQL_LAZYINDEX /tmp/sql2rcs > /dev/null
if [ $? -ne 0 ]; then
   echo "SQL_LAZYINDEX=1" >> /tmp/sql2rcs
   echo "SQL_LAZYINDEX=1" >> ../TRIPrcs.mods
fi

grep SQL_SELECTTEXT /tmp/sql2rcs > /dev/null
if [ $? -ne 0 ]; then
   echo "SQL_SELECTTEXT=0" >> /tmp/sql2rcs
   echo "SQL_SELECTTEXT=0" >> ../TRIPrcs.mods
fi

grep SQL_SELECTBLOB /tmp/sql2rcs > /dev/null
if [ $? -ne 0 ]; then
   echo "SQL_SELECTBLOB=0" >> /tmp/sql2rcs
   echo "SQL_SELECTBLOB=0" >> ../TRIPrcs.mods
fi


# Introduce changes
cp $CONFIGFILE $CONFIGFILE.sql201_bkp
mv /tmp/sql2rcs $CONFIGFILE
umask $OUMASK

echo ""
echo "Installation of TRIPsql $VERSION complete"
echo ""
